from uuid import (
    UUID,
)

from rest_framework.generics import (
    get_object_or_404,
)

from m3_rest_gar.util import (
    is_objectguid,
)


class GUIDLookupMixin:
    """
    Миксин для вьюсета добавляет возможность получать адресный объект как
    по objectid, так и по objectguid

    Если очень захотеть, можно сюда же впихнуть поиск по коду КЛАДРа
    Но лучше такого не хотеть, да
    """

    lookup_field = 'objectid'
    guid_lookup_field = 'objectguid'

    def get_object(self):
        queryset = self.filter_queryset(self.get_queryset())

        # Perform the lookup filtering.
        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field

        assert lookup_url_kwarg in self.kwargs, (
            'Expected view %s to be called with a URL keyword argument '
            'named "%s". Fix your URL conf, or set the `.lookup_field` '
            'attribute on the view correctly.' %
            (self.__class__.__name__, lookup_url_kwarg)
        )

        lookup_value = self.kwargs[lookup_url_kwarg]

        if is_objectguid(lookup_value):
            lookup_field = self.guid_lookup_field
        else:
            lookup_field = self.lookup_field

        filter_kwargs = {lookup_field: lookup_value}
        obj = get_object_or_404(queryset, **filter_kwargs)

        # May raise a permission denied
        self.check_object_permissions(self.request, obj)

        return obj



from collections.abc import (
    Iterable,
)
from uuid import (
    UUID,
)

from m3_gar.models.hierarchy import (
    Hierarchy,
)


# Пример административного деления
# 33634065 - д 54
# 210826 - ул Заречная
# 210984 - с Кунгуртуг
# 213492 - р-н Тере-Хольский
# 206101 - Респ Тыва

# Пример муниципального деления
# 33634065 - д 54
# 210826 - ул Заречная
# 210984 - с Кунгуртуг
# 95235279 - с.п. Шынаанский
# 95235278 - м.р-н Тере-Хольский
# 206101 - Респ Тыва


def get_hierarchy_models(hierarchy):
    """Возвращает список моделей иерархии по заданному коду (или нескольким кодам).

    Args:
        hierarchy: Код или список кодов иерархии.
            Примеры: 'adm', 'mun', 'any', ['adm', 'mun']

    Returns:
        Список классов моделей иерархии

    Raises:
        ValueError: неверный код иерархии
    """

    hierarchy_model_map = Hierarchy.get_shortname_map()

    if hierarchy == 'any':
        hierarchy = hierarchy_model_map.keys()
    elif isinstance(hierarchy, str):
        hierarchy = [hierarchy]
    elif isinstance(hierarchy, Iterable):
        pass
    else:
        raise ValueError(f'Invalid hierarchy value: {hierarchy}')

    try:
        hierarchy_models = [hierarchy_model_map[h] for h in hierarchy]
    except KeyError as e:
        raise ValueError(f'Invalid hierarchy value: {e}')

    return hierarchy_models


def is_objectguid(value):
    try:
        UUID(value)
    except ValueError:
        result = False
    else:
        result = True

    return result



import codecs
from pathlib import (
    Path,
)

from setuptools import (
    setup,
)


def read(fn):
    return codecs.open(Path(__file__).resolve().parent / fn).read()


setup(
    name='m3-rest-gar',
    version='1.0.41',
    description=("REST-service for GAR."),
    author='BARS Group',
    author_email='bars@bars.group',
    license="MIT",
    keywords="django rest gar",
    long_description=read('README.rst'),
    packages=['m3_rest_gar'],
    install_requires=read('requirements.txt'),
    classifiers=(
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
    ),
)
